// EkUtil.cpp: implementation file
//

#include "stdafx.h"

#include "EkUtil.h"

//=======================================================//
// Chapter 02: Documents and Document Templates          //
//=======================================================//

///////////////////////////////////////////////////////////
// CEkDocList: class implementation

CEkDocList::CEkDocList()
{
	CWinApp* pApp = AfxGetApp();
	ASSERT_VALID( pApp );

	// 1 - Iterate through the application's document
	// templates list
	POSITION posTemplate =
				   pApp->GetFirstDocTemplatePosition();
	while( posTemplate != NULL )
	{
		// 2 - For each document template object...
		CDocTemplate* pTemplate =
				pApp->GetNextDocTemplate( posTemplate );
		ASSERT_VALID( pTemplate );
		ASSERT_KINDOF( CDocTemplate, pTemplate );

		// 3 - Iterate through the template's document list
		POSITION posDocument =
				   pTemplate->GetFirstDocPosition();
		while( posDocument != NULL )
		{
			// 4 - For each document object...
			CDocument* pDoc =
					pTemplate->GetNextDoc( posDocument );
			ASSERT_VALID( pDoc );
			ASSERT_KINDOF( CDocument, pDoc );

			// 5 - Add the document pointer to our
			// internal list
			m_DocList.AddTail( pDoc );
		}
	}

	// 6 - Initialize our POSITION member
	m_posDocList = m_DocList.GetHeadPosition();
}

CEkDocList::~CEkDocList()
{
	m_DocList.RemoveAll();
	m_posDocList = NULL;
}

CDocument* CEkDocList::GetNextDoc()
{
	if( m_posDocList == NULL )
		return NULL;

	CDocument* pDoc = m_DocList.GetNext( m_posDocList );
	ASSERT_VALID( pDoc );
	ASSERT_KINDOF( CDocument, pDoc );

	return pDoc;
}

///////////////////////////////////////////////////////////
// EkCloseDocument: function implementation

void EkCloseDocument( CDocument* pDoc )
{
	ASSERT_VALID( pDoc );

	// 1 - Give the user a chance to save his document
	if( !pDoc->SaveModified() )
		return;

	// 2 - Destroy the document object
	pDoc->OnCloseDocument();
}

///////////////////////////////////////////////////////////
// EkGetActiveDocument: function implementation

CDocument* EkGetActiveDocument()
{
	// 1 - Get a pointer to the application's
	// main frame window
	CWnd* pWnd = AfxGetMainWnd();
	if( pWnd == NULL )
		return NULL;

	// 2 - Make sure the pointer is valid and more
	// strongly typed
	ASSERT_VALID( pWnd );
	ASSERT_KINDOF( CFrameWnd, pWnd );
	CFrameWnd* pMainFrame = static_cast< CFrameWnd* >( pWnd );

	// 3 - Get a pointer to the active frame window
	// (may be 'this' for SDI application)
	CFrameWnd* pActiveFrame = pMainFrame->GetActiveFrame();
	if( pActiveFrame == NULL )
		return NULL;

	// 4 - Return a pointer to the active document object
	return pActiveFrame->GetActiveDocument();
}

//=======================================================//
// Chapter 03: Views and Frame Windows                   //
//=======================================================//

///////////////////////////////////////////////////////////
// CEkFixedFormFrame: class implementation

IMPLEMENT_DYNCREATE(CEkFixedFormFrame, CMDIChildWnd)

CEkFixedFormFrame::CEkFixedFormFrame()
{
}

CEkFixedFormFrame::~CEkFixedFormFrame()
{
}

BEGIN_MESSAGE_MAP(CEkFixedFormFrame, CMDIChildWnd)
	//{{AFX_MSG_MAP(CEkFixedFormFrame)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CEkFixedFormFrame::PreCreateWindow(CREATESTRUCT& cs) 
{
	// Make the frame window non-resizable
	cs.style &= ~WS_THICKFRAME;
	cs.style &= ~WS_MAXIMIZEBOX;

	return CMDIChildWnd::PreCreateWindow(cs);
}

int CEkFixedFormFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CMDIChildWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// The child FormView should already have been created
	// by the call to CMDIChildWnd::OnCreate()
	CFormView* pFormView = static_cast< CFormView* >
								( GetDescendantWindow( AFX_IDW_PANE_FIRST, TRUE ) );
	ASSERT_VALID( pFormView );
	ASSERT_KINDOF( CFormView, pFormView );

	// Make sure that the view window size is correctly set
	RecalcLayout();

	// Make the view resize its parent frame -- that is
	// "us", normally !
	pFormView->ResizeParentToFit( FALSE );
	pFormView->ResizeParentToFit( TRUE );

	return 0;
}

///////////////////////////////////////////////////////////
// CEkLockableSplitter: class implementation

CEkLockableSplitter::CEkLockableSplitter()
{
	m_bLocked = FALSE;
}

CEkLockableSplitter::~CEkLockableSplitter()
{
}

BEGIN_MESSAGE_MAP(CEkLockableSplitter, CSplitterWnd)
	//{{AFX_MSG_MAP(CEkLockableSplitter)
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_SETCURSOR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CEkLockableSplitter::SetLock( BOOL bLock )
{
	m_bLocked = bLock;
}

BOOL CEkLockableSplitter::GetLock() const
{
	return m_bLocked;
}

void CEkLockableSplitter::ToggleLock()
{
	m_bLocked = !m_bLocked;
}

void CEkLockableSplitter::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if( m_bLocked )
	{
		// Bypass standard CSplitterWnd processing
		CWnd::OnLButtonDown(nFlags, point);
	}
	else
	{
		CSplitterWnd::OnLButtonDown(nFlags, point);
	}
}

void CEkLockableSplitter::OnMouseMove(UINT nFlags, CPoint point) 
{
	if( m_bLocked )
	{
		// Bypass standard CSplitterWnd processing
		CWnd::OnMouseMove(nFlags, point);
	}
	else
	{
		CSplitterWnd::OnMouseMove(nFlags, point);
	}
}

BOOL CEkLockableSplitter::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	if( m_bLocked )
	{
		// Bypass standard CSplitterWnd processing
		return CWnd::OnSetCursor(pWnd, nHitTest, message);
	}
	else
	{
		return CSplitterWnd::OnSetCursor(pWnd, nHitTest, message);
	}
}

///////////////////////////////////////////////////////////
// EkCreateNewWindow: function implementation

CFrameWnd* EkCreateNewWindow( CDocTemplate* pTemplate,
                              CDocument* pDocument )
{
	ASSERT_VALID( pTemplate );
	ASSERT_VALID( pDocument );

	// 1 - Create the new frame window
	// (will in turn create the associated view)
	CFrameWnd* pFrame = pTemplate->CreateNewFrame(
										pDocument, NULL );
	if( pFrame == NULL )
	{
		// Window creation failed
		TRACE0( "Warning: failed to create new frame.\n" );
		return NULL;
	}
	ASSERT_KINDOF( CFrameWnd, pFrame );

	// 2 - Tell the frame to update itself
	// (and its child windows)
	pTemplate->InitialUpdateFrame( pFrame, pDocument );

	// 3 - Return a pointer to the new frame window object
	return pFrame;
}

///////////////////////////////////////////////////////////
// EkGetActiveFrame: function implementation

CFrameWnd* EkGetActiveFrame()
{
	CWnd* pMainWnd = AfxGetMainWnd();
	ASSERT_VALID( pMainWnd );

	if( !pMainWnd->IsKindOf( RUNTIME_CLASS( CFrameWnd ) ) )
	{
		return NULL;
	}

	return static_cast<CFrameWnd*>( pMainWnd )
										->GetActiveFrame();
}

///////////////////////////////////////////////////////////
// EkGetActiveView: function implementation

CView* EkGetActiveView()
{
	CWnd* pMainWnd = AfxGetMainWnd();
	ASSERT_VALID( pMainWnd );

	if( !pMainWnd->IsKindOf( RUNTIME_CLASS( CFrameWnd ) ) )
	{
		return NULL;
	}

	return static_cast<CFrameWnd*>( pMainWnd )
						->GetActiveFrame()->GetActiveView();
}

///////////////////////////////////////////////////////////
// EkSwitchViewInFrame: function implementation

void EkSwitchViewInFrame( CFrameWnd* pFrame, CRuntimeClass* pViewClass )
{
	ASSERT_VALID( pFrame );
	ASSERT( pViewClass != NULL );

	ASSERT( pViewClass->IsDerivedFrom( RUNTIME_CLASS( CView ) ) );

	// 1 - Find the currently active view
	CView* pActiveView = pFrame->GetActiveView();
	if( pActiveView == NULL )
	{
		TRACE0( "Unable to switch: no active view\n" );
		return;
	}

	if( pActiveView->IsKindOf( pViewClass ) )
	{
		// No need to switch for same view class
		return;
	}

	// 2 - Store current view position
	CRect rcView;
	pActiveView->GetWindowRect( &rcView );

	// 3 - Find the associated document
	CDocument* pDoc = pActiveView->GetDocument();
	ASSERT_VALID( pDoc );

	// 4 - Make sure the document won't self-destruct
	// when active view is destroyed
	BOOL bSaveAutoDelete = pDoc->m_bAutoDelete;
	pDoc->m_bAutoDelete = FALSE;

	// 5 - Destroy the active view
	pActiveView->DestroyWindow();

	// 6 - Restore document to initial state
	pDoc->m_bAutoDelete = bSaveAutoDelete;

	// 7 - Initialize creation context used by CreateView()
	CCreateContext context;
	context.m_pNewDocTemplate = NULL;
	context.m_pLastView = NULL;
	context.m_pCurrentFrame = NULL;

	context.m_pNewViewClass = pViewClass;
	context.m_pCurrentDoc = pDoc;

	// 8 - Create the new view
	CView* pNewView = static_cast<CView*>
						( pFrame->CreateView( &context ) );

	ASSERT_VALID( pNewView );

	// 9 - Position the new view like the old one
	pFrame->ScreenToClient( &rcView );
	pNewView->MoveWindow( &rcView, TRUE );
	
	// 10 - Send WM_INITIALUPDATE to the view
	pFrame->InitialUpdateFrame( pDoc, TRUE );
}

///////////////////////////////////////////////////////////
// EkSwitchViewInSplitter: function implementation

void EkSwitchViewInSplitter(	CSplitterWnd* pSplitter,
								int row, int col,
								CRuntimeClass* pViewClass )
{
	ASSERT_VALID( pSplitter );
	ASSERT( pViewClass != NULL );

	ASSERT( pViewClass->
				IsDerivedFrom( RUNTIME_CLASS( CView ) ) );

	// 1 - Find the view to be replaced
	CWnd* pPaneWnd = pSplitter->GetPane( row, col );
	if( !pPaneWnd->IsKindOf( RUNTIME_CLASS( CView ) ) )
	{
		TRACE2(	"Unable to switch: pane (%d,%d) is not a view\n",
				row, col );
		return;
	}

	CView* pCurrentView = static_cast<CView*>( pPaneWnd );
	ASSERT_VALID( pCurrentView );
	ASSERT_KINDOF( CView, pCurrentView );

	if( pCurrentView->IsKindOf( pViewClass ) )
	{
		// No need to switch for same view class
		return;
	}

	// 2 - Store current view position and activation state
	CRect rcView;
	pCurrentView->GetWindowRect( &rcView );

	CView* pActiveView = pSplitter->
						GetParentFrame()->GetActiveView();
	BOOL bSaveActive = ( pActiveView == NULL ) 
						|| ( pActiveView == pCurrentView );

	// 3 - Find the associated document
	CDocument* pDoc = pCurrentView->GetDocument();
	ASSERT_VALID( pDoc );

	// 4 - Make sure the document won't self-destruct
	// when current view is destroyed
	BOOL bSaveAutoDelete = pDoc->m_bAutoDelete;
	pDoc->m_bAutoDelete = FALSE;

	// 5 - Destroy the current view
	pCurrentView->DestroyWindow();

	// 6 - Restore document to initial state
	pDoc->m_bAutoDelete = bSaveAutoDelete;

	// 7 - Initialize creation context used by CreateView()
	CCreateContext context;
	context.m_pNewDocTemplate = NULL;
	context.m_pLastView = NULL;
	context.m_pCurrentFrame = NULL;

	context.m_pNewViewClass = pViewClass;
	context.m_pCurrentDoc = pDoc;

	// 8 - Create the new view
	pSplitter->CreateView(	row, col, pViewClass,
							rcView.Size(), &context );

	CView* pNewView = static_cast<CView*>
						( pSplitter->GetPane( row, col ) );
	ASSERT_VALID( pNewView );
	ASSERT_KINDOF( CView, pNewView );

	// 9 - Position the new view like the old one and
	// activate it if needed
	pSplitter->ScreenToClient( &rcView );
	pNewView->MoveWindow( &rcView, TRUE );
	if( bSaveActive )
	{
		pSplitter->GetParentFrame()->SetActiveView( pNewView );
	}
	
	// 10 - Send WM_INITIALUPDATE to the view
	pNewView->GetParentFrame()->InitialUpdateFrame( pDoc, TRUE );
}

//=======================================================//
// Chapter 04: Dialog Boxes                              //
//=======================================================//

///////////////////////////////////////////////////////////
// EkDrawBitmap: function implementation

void EkDrawBitmap( CDC* pDC, CBitmap* pBitmap, CRect rect, BOOL bCenter /* = TRUE */ )
{
	ASSERT_VALID( pDC );
	ASSERT_VALID( pBitmap );

   // 1 - Create compatible memory DC
   CDC dcMem;
   dcMem.CreateCompatibleDC( pDC );

   // 2 - Select bitmap into memory DC
   CBitmap* pOldBitmap = dcMem.SelectObject( pBitmap );

   if( bCenter )
   {
	   // 3 - Center bitmap in destination rectangle
	   BITMAP bitmap;
	   pBitmap->GetObject( sizeof( BITMAP ), &bitmap);
	   CSize sizeBitmap( bitmap.bmWidth, bitmap.bmHeight );
	   CSize diff = rect.Size() - sizeBitmap;
	   rect.DeflateRect( diff.cx / 2, diff.cy / 2 );
   }

   // 4 - Blit the bitmap from the memory DC to the real DC
   pDC->BitBlt( rect.left, rect.top,
             rect.Width(), rect.Height(),
             &dcMem, 0, 0, SRCCOPY );

   // 5 - Deselect bitmap from memory DC
   dcMem.SelectObject( pOldBitmap );
}

///////////////////////////////////////////////////////////
// EkExpandDialog: function implementation

void EkExpandDialog( CDialog* pDlg, UINT nSmallID, UINT nLargeID, UINT nButtonID )
{
	ASSERT_VALID( pDlg );

	// 1 - Get the More/Less button control
	CWnd* pWndButton = pDlg->GetDlgItem( nButtonID );
	ASSERT_VALID( pWndButton );

	// 2 - Load button resource string and parse More/Less parts
	CString strButton;
	VERIFY( strButton.LoadString( nButtonID ) );

	CString strMore, strLess;
	AfxExtractSubString( strMore, strButton, 0, _T( '\n' ) );
	AfxExtractSubString( strLess, strButton, 1, _T( '\n' ) );

	// 3 - Find out if we need to expand or collapse the dialog
	CString strCaption;
	pWndButton->GetWindowText( strCaption );
	BOOL bExpand = ( strCaption == strMore );	// Collapse by default

	// 4 - Get current dialog window rectangle
	CRect rcDialog;
	pDlg->GetWindowRect( &rcDialog );

	int nNewHeight = -1;

	if( bExpand )
	{
		// 5a - Change More/Less button caption
		pWndButton->SetWindowText( strLess );

		// 6a - Calculate new dialog height
		CWnd* pWndLarge = pDlg->GetDlgItem( nLargeID );
		ASSERT_VALID( pWndLarge );

		CRect rcLarge;
		pWndLarge->GetWindowRect( &rcLarge );

		nNewHeight = rcLarge.top-rcDialog.top;
	}
	else
	{
		// 5b - Change More/Less button caption
		pWndButton->SetWindowText( strMore );

		// 6b - Calculate new dialog height
		CWnd* pWndSmall = pDlg->GetDlgItem( nSmallID );
		ASSERT_VALID( pWndSmall );

		CRect rcSmall;
		pWndSmall->GetWindowRect( &rcSmall );

		nNewHeight = rcSmall.top-rcDialog.top;
	}

	// 7 - Set new dialog height
	ASSERT( nNewHeight > 0 );
	pDlg->SetWindowPos( NULL, 0, 0,
						rcDialog.Width(), nNewHeight,
						SWP_NOMOVE | SWP_NOZORDER );

	// 8 - Set the enabled state for each control depending on whether
	// the control is currently visible or not
	CWnd* pWndControl = pDlg->GetWindow( GW_CHILD );
	while( pWndControl != NULL )
	{
		CRect rcControl;
		pWndControl->GetWindowRect( &rcControl );
		pWndControl->EnableWindow( rcControl.top <= rcDialog.top + nNewHeight );
		pWndControl = pWndControl->GetWindow( GW_HWNDNEXT );
	}

	// 9 - Check if a control still has the focus
	// (can lose it if the active control becomes disabled)
	CWnd* pWndActiveControl = CWnd::GetFocus();
	if( pWndActiveControl == NULL )
	{
		// 10 - Set focus to "first" control on dialog
		CWnd* pWndFirstControl = pDlg->GetNextDlgTabItem( NULL );
		ASSERT_VALID( pWndFirstControl );
		ASSERT( pWndFirstControl->IsWindowEnabled() );
		pWndFirstControl->SetFocus();
	}
}

//=======================================================//
// Chapter 05: Property Sheets                           //
//=======================================================//

///////////////////////////////////////////////////////////
// EkCenterPropertySheetButtons: function implementation

void EkCenterPropertySheetButtons(	CPropertySheet* pPSheet,
									BOOL bCenterOK,
									BOOL bCenterCancel,
									BOOL bCenterApply )
{
	ASSERT_VALID( pPSheet );
	ASSERT( bCenterOK || bCenterCancel || bCenterApply );

	// Find "OK" button rectangle
	CWnd* pWndOK = pPSheet->GetDlgItem( IDOK );
	ASSERT( pWndOK != NULL );
	CRect rcOK;
	pWndOK->GetWindowRect( &rcOK );
	pPSheet->ScreenToClient( &rcOK );

	// Find "Cancel" button rectangle
	CWnd* pWndCancel = pPSheet->GetDlgItem( IDCANCEL );
	ASSERT( pWndCancel != NULL );
	CRect rcCancel;
	pWndCancel->GetWindowRect( &rcCancel );
	pPSheet->ScreenToClient( &rcCancel );

	// Find "Apply" button rectangle
	CWnd* pWndApply = pPSheet->GetDlgItem( ID_APPLY_NOW );
	ASSERT( pWndApply != NULL );
	CRect rcApply;
	pWndApply->GetWindowRect( &rcApply );
	pPSheet->ScreenToClient( &rcApply );

	// Find property sheet client rectangle
	CRect rcClient;
	pPSheet->GetClientRect( &rcClient );

	// Compute layout values
	int nButtonWidth = rcOK.Width();
	int nButtonMargin = rcCancel.left - rcOK.right;
	int nButtons = (bCenterOK ? 1 : 0) + (bCenterCancel ? 1 : 0)
						+ (bCenterApply ? 1 : 0);
	int nGlobalWidth = nButtonWidth * nButtons;
	if( nButtons > 1 )
	{
		nGlobalWidth += nButtonMargin * (nButtons - 1);
	}

	int nCurrentX = ( rcClient.left + rcClient.right - nGlobalWidth ) / 2;
	int nTop = rcOK.top;

	// Center "OK" button
	if( bCenterOK )
	{
		pWndOK->SetWindowPos(		NULL,
									nCurrentX,
									nTop,
									0, 0,
									SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );

		nCurrentX += nButtonWidth + nButtonMargin;
	}

	// Center "Cancel" button
	if( bCenterCancel )
	{
		pWndCancel->SetWindowPos(	NULL,
									nCurrentX,
									nTop,
									0, 0,
									SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );

		nCurrentX += nButtonWidth + nButtonMargin;
	}

	// Center "Apply" button
	if( bCenterApply )
	{
		pWndApply->SetWindowPos(	NULL,
									nCurrentX,
									nTop,
									0, 0,
									SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );
	}
}

///////////////////////////////////////////////////////////
// EkCreateEmbeddedPropertySheet: function implementation

BOOL EkCreateEmbeddedPropertySheet(	CWnd* pParent,
									CPropertySheet* pPSheet,
									DWORD dwStyle /* = WS_CHILD | WS_VISIBLE */,
									DWORD dwExStyle /* = 0 */ )
{
	ASSERT_VALID( pParent );
	ASSERT_VALID( pPSheet );

	// 1 - Create the embedded property sheet window
	if( !pPSheet->Create( pParent,  dwStyle, dwExStyle ) )
	{
		TRACE0( "Embedded property sheet creation failed\n" );
		return FALSE;
	}

	// 2 - Add WS_TABSTOP and WS_EX_CONTROLPARENT to the property sheet styles
	pPSheet->ModifyStyle( 0, WS_TABSTOP );
	pPSheet->ModifyStyleEx ( 0, WS_EX_CONTROLPARENT );

	// 3 - Add WS_EX_CONTROLPARENT to the parent window styles
	pParent->ModifyStyleEx ( 0, WS_EX_CONTROLPARENT );

	return TRUE;
}

///////////////////////////////////////////////////////////
// EkHidePropertySheetButtons: function implementation

void EkHidePropertySheetButtons(	CPropertySheet* pPSheet,
									BOOL bHideOK,
									BOOL bHideCancel, 
									BOOL bHideApply )
{
	ASSERT_VALID( pPSheet );

	// Hide "OK" button
	if( bHideOK )
	{
		CWnd* pWnd = pPSheet->GetDlgItem( IDOK );
		ASSERT( pWnd != NULL );
		pWnd->ShowWindow( SW_HIDE );
	}

	// Hide "Cancel" button
	if( bHideCancel )
	{
		CWnd* pWnd = pPSheet->GetDlgItem( IDCANCEL );
		ASSERT( pWnd != NULL );
		pWnd->ShowWindow( SW_HIDE );
	}

	// Hide "Apply" button
	if( bHideApply )
	{
		CWnd* pWnd = pPSheet->GetDlgItem( ID_APPLY_NOW );
		ASSERT( pWnd != NULL );
		pWnd->ShowWindow( SW_HIDE );
	}
}

///////////////////////////////////////////////////////////
// EkPositionEmbeddedPropertySheet: function implementation

void EkPositionEmbeddedPropertySheet(	CWnd* pParent,
										CPropertySheet* pPSheet,
										CRect rcNewPosition )
{
	ASSERT_VALID( pParent );
	ASSERT_VALID( pPSheet );

	// 1 - Get current coordinates of tab control
	// and property sheet window
	CTabCtrl* pTabCtrl = pPSheet->GetTabControl();
	ASSERT( pTabCtrl != NULL );

	CRect rcTabCtrl;
	pTabCtrl->GetWindowRect( &rcTabCtrl );
	pParent->ScreenToClient( &rcTabCtrl );

	CRect rcPSheet;
	pPSheet->GetWindowRect( &rcPSheet );
	pParent->ScreenToClient( &rcPSheet );

	// 2 - Calculate margin between property sheet
	// and tab control
	int dcx = rcPSheet.Width() - rcTabCtrl.Width();
	int dcy = rcPSheet.Height() - rcTabCtrl.Height();

	// 3 - Move and resize property sheet window
	// (also moves the tab window because it is a child
	// of the property sheet window)
	pPSheet->MoveWindow( 	rcNewPosition .left, rcNewPosition.top,
							rcNewPosition .Width(), rcNewPosition.Height() );

	// 4 - Resize tab control window to restore
	// right / bottom margins
	pTabCtrl->SetWindowPos(	NULL,
							0, 0,
							rcNewPosition.Width() - dcx,
							rcNewPosition.Height() - dcy, 
							SWP_NOZORDER | SWP_NOMOVE | SWP_NOACTIVATE );

	// 5 - Activate each property page to prevent drawing
	// problem
	int nCurrentPage = pPSheet->GetActiveIndex();
	for( int i = 0; i < pPSheet->GetPageCount(); ++i )
	{
		pPSheet->SetActivePage( i );
	}

	pPSheet->SetActivePage( nCurrentPage );
}

void EkPositionEmbeddedPropertySheet(	CWnd* pParent,
										CPropertySheet* pPSheet,
										UINT nIDPSheetArea )
{
	ASSERT_VALID( pParent );
	ASSERT_VALID( pPSheet );

	// 1 - Retrieve property sheet destination position
	CRect rcNewPosition;
	CWnd* pWndNewArea = pParent->GetDlgItem( nIDPSheetArea );
	if( pWndNewArea == NULL )
	{
		ASSERT( FALSE );	// Invalid nIDPSheetArea
		return;
	}

	pWndNewArea->GetWindowRect( &rcNewPosition );
	pParent->ScreenToClient( &rcNewPosition );

	// 2 - Call overloaded function
	EkPositionEmbeddedPropertySheet( pParent, pPSheet, rcNewPosition );
}

///////////////////////////////////////////////////////////
// EkResizePropertySheet: function implementation

void EkResizePropertySheet( CPropertySheet* pPSheet, int nWidth )
{
	ASSERT_VALID( pPSheet );

	// 1 - Get current dimensions of tab control
	// and property sheet window
	CTabCtrl* pTabCtrl = pPSheet->GetTabControl();
	ASSERT( pTabCtrl != NULL );

	CRect rcTabCtrl;
	pTabCtrl->GetWindowRect( &rcTabCtrl );
	pPSheet->ScreenToClient( &rcTabCtrl );

	CRect rcPSheet;
	pPSheet->GetWindowRect( &rcPSheet );

	// 2 - Compute resizing offset
	int dcx = rcPSheet.Width() - nWidth;

	// 3 - Resize property sheet window
	pPSheet->SetWindowPos(	NULL,
							0, 0,
							nWidth,
							rcPSheet.Height(), 
							SWP_NOZORDER | SWP_NOMOVE | SWP_NOACTIVATE );


	// 4 - Resize tab control window to restore
	// right margin
	pTabCtrl->SetWindowPos(	NULL,
							0, 0,
							rcTabCtrl.Width() - dcx,
							rcTabCtrl.Height(), 
							SWP_NOZORDER | SWP_NOMOVE | SWP_NOACTIVATE );

	// 5 - Activate each property page to prevent drawing
	// problem
	int nCurrentPage = pPSheet->GetActiveIndex();
	for( int i = 0; i < pPSheet->GetPageCount(); ++i )
	{
		pPSheet->SetActivePage( i );
	}

	pPSheet->SetActivePage( nCurrentPage );
}

//=======================================================//
// Chapter 06: Toolbars and Status Bars                  //
//=======================================================//

/////////////////////////////////////////////////////////////////////////////
// CEkDrawStatusBar: class implementation

#include <afxpriv.h>  // For WM_IDLEUPDATECMDUI message

CEkDrawStatusBar::CEkDrawStatusBar()
{
}

CEkDrawStatusBar::~CEkDrawStatusBar()
{
}

BEGIN_MESSAGE_MAP(CEkDrawStatusBar, CStatusBar)
	//{{AFX_MSG_MAP(CEkDrawStatusBar)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CEkDrawStatusBar::OnPaint() 
{
	// 1 - Let base class paint itself
	CStatusBar::OnPaint();

	// 2 - Send a WM_IDLEUPDATECMDUI message to ourself
	// to ensure our "drawing" panes repaint correctly
	SendMessage( WM_IDLEUPDATECMDUI );
}

///////////////////////////////////////////////////////////
// EkCreateProgressBar: function implementation

BOOL EkCreateProgressBar(	CProgressCtrl* pProgressCtrl,
							CStatusBar* pStatusBar,
							LPCTSTR szMessage /* = NULL */,
							int nPaneIndex /* = 0 */,
							int cxMargin /* = 10 */,
							int cxMaxWidth /* = -1 */,
							UINT nIDControl /* = 1 */ )
{
	ASSERT_VALID( pProgressCtrl );
	ASSERT_VALID( pStatusBar );

	// 1 - Calculate destination rectangle for progress control
    CRect rc;
    pStatusBar->GetItemRect( nPaneIndex, &rc );

	// 2 - Define progress bar horizontal offset
	if( szMessage != NULL )
	{
		// Compute message text extent
		CClientDC dc( pStatusBar );
		CFont* pFont = pStatusBar->GetFont();
		CFont* pOldFont = dc.SelectObject( pFont );
		CSize sizeText = dc.GetTextExtent( szMessage );
		dc.SelectObject( pOldFont );

		rc.left += sizeText.cx + cxMargin;
	}

	// 3 - Compute progress bar width
	if( cxMaxWidth != -1 )
	{
		rc.right = rc.left + min( cxMaxWidth, rc.Width() );
	}

	// 4 - Display message text
	pStatusBar->SetPaneText( nPaneIndex, szMessage );
	pStatusBar->RedrawWindow();

	// 5 - Create progress control
    return pProgressCtrl->Create(	WS_CHILD | WS_VISIBLE,
									rc, pStatusBar, nIDControl );
}

///////////////////////////////////////////////////////////
// EkCreateToolBar: function implementation

BOOL EkCreateToolBar(	CFrameWnd* pParentFrame,
						CToolBar* pBar,
						UINT nIDBar,
						UINT nIDResource /* = 0 */,
						DWORD dwStyle1 /* = WS_CHILD | WS_VISIBLE | CBRS_TOP */,
						DWORD dwStyle2 /* = CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC */ )
{
	ASSERT_VALID( pParentFrame );
	ASSERT_VALID( pBar );

	// 1 - No IDResource means resource ID is same as bar ID
	if( nIDResource == 0 )
	{
		nIDResource = nIDBar;
	}

	// 2 - Create toolbar and load resources
	if( !pBar->Create( pParentFrame, dwStyle1, nIDBar ) ||
		!pBar->LoadToolBar( nIDResource ) )
	{
		TRACE1( "Failed to create toolbar with ID=%d\n", nIDBar );
		return FALSE;      // fail to create
	}

	// 3 - Define toolbar styles
	pBar->SetBarStyle( pBar->GetBarStyle() | dwStyle2 );

	return TRUE;
}

///////////////////////////////////////////////////////////
// EkDockBarNextTo: function implementation

void EkDockBarNextTo(	CControlBar* pNewBar,
						CControlBar* pDockedBar,
						UINT nDockBarID /* = AFX_IDW_DOCKBAR_TOP */ )
{
	ASSERT_VALID( pDockedBar );
	ASSERT_VALID( pNewBar );

	// 1 - Find the frame where we will dock
	CFrameWnd* pFrame = pDockedBar->GetDockingFrame();

	// 2 - Force MFC to compute the positions
	// of the docked control bar(s)
	pFrame->RecalcLayout();

	// 3 - Compute rectangle of already "docked" bar
	CRect rect;
	pDockedBar->GetWindowRect( &rect );

	// 4 - Offset the rectangle slightly to the bottom right
	// so that the new bar will dock either to the right or
	// to the bottom of the existing bar (depending on the
	// side where this last bar is already docked)
	rect.OffsetRect(1,1);

	// 5 - Dock new bar to specified position
	pFrame->DockControlBar( pNewBar, nDockBarID, &rect );
}

///////////////////////////////////////////////////////////
// EkDockToolBar: function implementation

void EkDockToolBar(	CFrameWnd* pParentFrame,
					CToolBar* pBar,
					UINT nDockBarID /* = AFX_IDW_DOCKBAR_TOP */,
					CControlBar* pBarNextTo /* = NULL */,
					DWORD dwStyleDocking /* = CBRS_ALIGN_ANY */ )
{
	ASSERT_VALID( pParentFrame );
	ASSERT_VALID( pBar );

	ASSERT( ::IsWindow( pBar->GetSafeHwnd() ) );

	// 1 - Define toolbar docking behavior
	pBar->EnableDocking( dwStyleDocking );

	if( pBarNextTo != NULL )
	{
		// 2a - Dock toolbar next to an existing bar
		EkDockBarNextTo( pBar, pBarNextTo, nDockBarID );
	}
	else
	{
		// 2b - Dock toolbar on its own dockbar
		pParentFrame->DockControlBar( pBar, nDockBarID );
	}
}

///////////////////////////////////////////////////////////
// EkIsBarVisible: function implementation

BOOL EkIsBarVisible( CControlBar* pBar )
{
	ASSERT_VALID( pBar );

	return ( ( pBar->GetStyle() & WS_VISIBLE ) != 0 );
}

///////////////////////////////////////////////////////////
// EkSetToolBarButtonText: function implementation

void EkSetToolBarButtonText( CToolBar* pBar, UINT nIDStrings, TCHAR chSep /* = _T( '\n' ) */ )
{
	ASSERT_VALID( pBar );
	
	// 1 - Load string from resource
	CString strButtons;
	VERIFY( strButtons.LoadString( nIDStrings ) );

	// 2 - Start at beginning of string
	int nIndex = 0;
	LPCTSTR pszStart = strButtons;
	LPCTSTR pszEnd = NULL;

	// 3 - Isolate each substring and set button text
	do
	{
		// 4 - Find next separator
		pszEnd = _tcschr( pszStart, chSep );

		int nLen = ( pszEnd == NULL ) ?
						_tcslen( pszStart ) : (int)( pszEnd - pszStart );
		ASSERT( nLen >= 0 );

		// 5 - Create CString from substring
		CString strSubString;
		memcpy(	strSubString.GetBufferSetLength( nLen ),
				pszStart, nLen * sizeof( TCHAR ) );

		// 6 - Set button text, bump index
		pBar->SetButtonText( nIndex++, strSubString );

		// 7 - Skip separator
		pszStart += nLen + 1;
	}
	while( pszEnd != NULL );
}

///////////////////////////////////////////////////////////
// EkSwitchBars: function implementation

void EkSwitchBars(	CFrameWnd* pFrame,
					CControlBar* pBar1,
					CControlBar* pBar2,
					BOOL bShowBar2,
					BOOL bDelay /* = TRUE */ )
{
	ASSERT_VALID( pFrame );
	ASSERT_VALID( pBar1 );
	ASSERT_VALID( pBar2 );
	
	// 1 - Compute "From" and "To" bars
	CControlBar* pBarFrom = bShowBar2 ? pBar1 : pBar2;
	CControlBar* pBarTo = bShowBar2 ? pBar2 : pBar1;
	BOOL bVisible = EkIsBarVisible( pBarFrom );

	// 2 - Exchange "From" and "To" window IDs
	UINT nIDFrom = ::GetWindowLong( pBarFrom->GetSafeHwnd(), GWL_ID );
	UINT nIDTo = ::GetWindowLong( pBarTo->GetSafeHwnd(), GWL_ID );

	::SetWindowLong( pBarFrom->GetSafeHwnd(), GWL_ID, nIDTo );
	::SetWindowLong( pBarTo->GetSafeHwnd(), GWL_ID, nIDFrom );

	// 3 - Hide "From" bar, show "To" bar if "From" bar was visible
	pFrame->ShowControlBar( pBarFrom, FALSE, bDelay );
	pFrame->ShowControlBar( pBarTo, bVisible, bDelay );
}

//=======================================================//
// Chapter 07: Menus                                     //
//=======================================================//

///////////////////////////////////////////////////////////
// EkUpdateMenuUI: function implementation

void EkUpdateMenuUI( CWnd* pOwner, CMenu* pMenu, BOOL bAutoMenuEnable /* = TRUE */ )
{
	// Checks the enabled/checked state of various menu items
	// (adapted from MFC's own CFrameWnd::OnInitMenuPopup() function --
	// WinFrm.cpp)

	ASSERT_VALID( pOwner );
	ASSERT( pMenu != NULL );

	// Create and initialize the famous CCmdUI object
	CCmdUI state;
	state.m_pMenu = pMenu;
	ASSERT(state.m_pOther == NULL);
	ASSERT(state.m_pParentMenu == NULL);

	// determine if menu is popup in top-level menu and set m_pOther to
	//  it if so (m_pParentMenu == NULL indicates that it is secondary popup)
	HMENU hParentMenu;
	if (AfxGetThreadState()->m_hTrackingMenu == pMenu->m_hMenu)
		state.m_pParentMenu = pMenu;    // parent == child for tracking popup
	else if ((hParentMenu = ::GetMenu(pOwner->m_hWnd)) != NULL)
	{
		CWnd* pParent = pOwner->GetTopLevelParent();
			// child windows don't have menus -- need to go to the top!
		if (pParent != NULL &&
			(hParentMenu = ::GetMenu(pParent->m_hWnd)) != NULL)
		{
			int nIndexMax = ::GetMenuItemCount(hParentMenu);
			for (int nIndex = 0; nIndex < nIndexMax; nIndex++)
			{
				if (::GetSubMenu(hParentMenu, nIndex) == pMenu->m_hMenu)
				{
					// when popup is found, m_pParentMenu is containing menu
					state.m_pParentMenu = CMenu::FromHandle(hParentMenu);
					break;
				}
			}
		}
	}

	state.m_nIndexMax = pMenu->GetMenuItemCount();

	// For each menu item...
	for (state.m_nIndex = 0; state.m_nIndex < state.m_nIndexMax;
	  state.m_nIndex++)
	{
		// Get menu item ID
		state.m_nID = pMenu->GetMenuItemID(state.m_nIndex);
		if (state.m_nID == 0)
			continue; // menu separator or invalid cmd - ignore it

		ASSERT(state.m_pOther == NULL);
		ASSERT(state.m_pMenu != NULL);
		if (state.m_nID == (UINT)-1)
		{
			// Maybe a popup menu, route to first item of
			// that popup
			state.m_pSubMenu = pMenu->GetSubMenu(state.m_nIndex);
			if (state.m_pSubMenu == NULL ||
				(state.m_nID = state.m_pSubMenu->GetMenuItemID(0)) == 0 ||
				state.m_nID == (UINT)-1)
			{
				continue;       // first item of popup can't be routed to
			}
			state.DoUpdate(pOwner, FALSE);    // popups are never auto disabled
		}
		else
		{
			// Normal menu item:
			// Auto enable/disable if 'bAutoMenuEnable' argument is set
			// and command is _not_ a system command
			state.m_pSubMenu = NULL;
			state.DoUpdate(pOwner, bAutoMenuEnable && state.m_nID < 0xF000);
		}

		// adjust for menu deletions and additions
		UINT nCount = pMenu->GetMenuItemCount();
		if (nCount < state.m_nIndexMax)
		{
			state.m_nIndex -= (state.m_nIndexMax - nCount);
			while (state.m_nIndex < nCount &&
				pMenu->GetMenuItemID(state.m_nIndex) == state.m_nID)
			{
				state.m_nIndex++;
			}
		}
		state.m_nIndexMax = nCount;
	}
}


//=======================================================//
// Chapter 08: Printing and print preview               //
//=======================================================//

///////////////////////////////////////////////////////////
// CEkPrintingDlg: class implementation

CEkPrintingDlg::CEkPrintingDlg(CWnd* pParent, CWnd* pStandardDialog)
	: CDialog(CEkPrintingDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CEkPrintingDlg)
	//}}AFX_DATA_INIT

	ASSERT_VALID( pStandardDialog );
	m_pStandardDialog = pStandardDialog;

	// Create modeless dialog box
	// (MFC automatically disables the application's
	// main window while printing)
	Create(CEkPrintingDlg::IDD, pParent);
}


void CEkPrintingDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEkPrintingDlg)
	DDX_Control(pDX, AFX_IDC_PRINT_DOCNAME, m_DocName);
	DDX_Control(pDX, AFX_IDC_PRINT_PRINTERNAME, m_PrinterName);
	DDX_Control(pDX, AFX_IDC_PRINT_PORTNAME, m_PortName);
	DDX_Control(pDX, AFX_IDC_PRINT_PAGENUM, m_PageNum);
	DDX_Control(pDX, IDC_PROGRESS, m_Progress);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEkPrintingDlg, CDialog)
	//{{AFX_MSG_MAP(CEkPrintingDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


BOOL CEkPrintingDlg::OnInitDialog() 
{
	SetWindowText( AfxGetAppName() );
	CenterWindow();
	
	return CDialog::OnInitDialog();
}


void CEkPrintingDlg::OnCancel() 
{
	ASSERT_VALID( m_pStandardDialog );
	ASSERT( ::IsWindow( m_pStandardDialog->GetSafeHwnd() ) );

	// Ask MFC's "Printing..." dialog to cancel
	// the printing process
	m_pStandardDialog->SendMessage( WM_COMMAND, IDCANCEL, 0 );
}

///////////////////////////////////////////////////////////
// EkChangePrintingOrientation: function implementation

void EkChangePrintingOrientation( CDC* pDC, CPrintInfo* pInfo, short NewOrientation )
{
	ASSERT_VALID( pDC );
	ASSERT( pInfo != NULL );

	DEVMODE* pDevMode = pInfo->m_pPD->GetDevMode();

	// Only change if new orientation is different 
	if( pDevMode->dmOrientation != NewOrientation )
	{
		pDevMode->dmOrientation = NewOrientation;
		pDC->ResetDC( pDevMode );
	}
}

///////////////////////////////////////////////////////////
// EkColorRefToGray: function implementation

COLORREF EkColorRefToGray( COLORREF crColor )
{
	// Extract (r, g, b) factors from COLORREF
	BYTE red = GetRValue( crColor );
	BYTE green = GetGValue( crColor );
	BYTE blue = GetBValue( crColor );

	// Convert (r, g, b) to gray using standard formula
	BYTE gray = static_cast< BYTE >( ( red*30 + green*59 + blue*11 ) / 100 );
	return RGB( gray, gray, gray );
}

///////////////////////////////////////////////////////////
// EkCreateDefaultPrinterDC: function implementation

BOOL EkCreateDefaultPrinterDC( CDC* pDC )
{
	ASSERT_VALID( pDC );

	PRINTDLG pd;
	AfxGetApp()->GetPrinterDeviceDefaults( &pd );
	return AfxGetApp()->CreatePrinterDC( *pDC );
}

///////////////////////////////////////////////////////////
// EkGetPrintMode: function implementation

/* enum EK_PRINT_MODE { DISPLAYING, PRINTING, PREVIEWING }; */

EK_PRINT_MODE EkGetPrintMode( CPrintInfo* pInfo )
{
	if( pInfo == NULL )
	{
		return DISPLAYING;
	}

	if( pInfo->m_bPreview )
	{
		return PREVIEWING;
	}

	return PRINTING;
}

EK_PRINT_MODE EkGetPrintMode( CDC* pDC )
{
	ASSERT_VALID( pDC );

	if( !pDC->IsPrinting() )
	{
		return DISPLAYING;
	}

	if( pDC->m_hDC != pDC->m_hAttribDC )
	{
		return PREVIEWING;
	}

	return PRINTING;
}

///////////////////////////////////////////////////////////
// EkIsMonoPreview: function implementation

BOOL EkIsMonoPreview( CDC* pDC )
{
	ASSERT_VALID( pDC );

	// Check for monochrome device context
	BOOL bMono = ( pDC->GetDeviceCaps( NUMCOLORS ) == 2 );

	// Check for print preview mode
	BOOL bPreview = ( pDC->m_hDC != pDC->m_hAttribDC );

	return( bMono && bPreview );
}

